@@ -190,6 +190,10 @@ class Push(Interface):
             recursive=False,
             recursion_limit=None,
             jobs=None):
+        # push uses '^' to annotate the previous pushed committish, and None for default
+        # behavior. '' was/is (to be deprecated) used in `publish`. Alert user about the mistake
+        if since == '':
+            raise ValueError("'since' should point to commitish or use '^'.")
         # we resolve here, because we need to perform inspection on what was given
         # as an input argument further down
         paths = [resolve_path(p, dataset) for p in assure_list(path)]
@@ -216,6 +220,7 @@ class Push(Interface):
             yield dict(
                 res_kwargs,
                 status='error',
+                path=ds.path,
                 message="Unknown push target '{}'. {}".format(
                     to,
                     'Known targets: {}.'.format(', '.join(repr(s) for s in sr))
@@ -393,9 +398,14 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
         label='Push',
         total=4,
     )
-    if not target:
+    # pristine input arg
+    _target = target
+    # verified or auto-detected
+    target = None
+    if not _target:
         try:
             # let Git figure out what needs doing
+            # we will reuse the result further down again, so nothing is wasted
             wannabe_gitpush = repo.push(remote=None, git_options=['--dry-run'])
             # we did not get an explicit push target, get it from Git
             target = set(p.get('remote', None) for p in wannabe_gitpush)
@@ -432,13 +442,15 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
             # can only be a single one at this point
             target = target.pop()
 
-    if target not in repo.get_remotes():
-        yield dict(
-            res_kwargs,
-            status='error',
-            message=(
-                "Unknown target sibling '%s'.", target))
-        return
+    if not target:
+        if _target not in repo.get_remotes():
+            yield dict(
+                res_kwargs,
+                status='error',
+                message=(
+                    "Unknown target sibling '%s'.", _target))
+            return
+        target = _target
 
     log_progress(
         lgr.info, pbar_id, "Push refspecs",
@@ -461,9 +473,12 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
     # do this on the main target only, and apply the result to all
     # dependencies
     try:
-        wannabe_gitpush = repo.push(
-            remote=target,
-            git_options=['--dry-run'])
+        if _target:
+            # only do it when an explicit target was given, otherwise
+            # we can reuse the result from the auto-probing above
+            wannabe_gitpush = repo.push(
+                remote=target,
+                git_options=['--dry-run'])
     except Exception as e:
         lgr.debug(
             'Dry-run push to check push configuration failed, '
@@ -544,54 +559,28 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
     # and lastly the primary push target
     target_is_git_remote = repo.config.get(
         'remote.{}.url'.format(target), None) is not None
-    # only attempt, if Git knows about a URL, otherwise this is
-    # a pure special remote that doesn't deal with the git repo
-    if target_is_git_remote:
-        # push the main branches of interest first, but not yet (necessarily)
-        # the git-annex branch. We ant to push first in order to hit any
-        # conflicts or unknown history before we move data. Otherwise out
-        # decision making done above (--since ...) might have been
-        # inappropriate.
-        push_ok = True
-        for p in _push_refspecs(
-                repo,
-                target,
-                refspecs2push,
-                force_git_push,
-                res_kwargs.copy()):
-            if p['status'] not in ('ok', 'notneeded'):
-                push_ok = False
-            yield p
-        if not push_ok:
-            # error-type results have been yielded, the local status quo is
-            # outdated/invalid, stop to let user decide how to proceed.
-            # TODO final global error result for the dataset?!
-            return
 
     # git-annex data copy
     #
-    if not is_annex_repo:
+    if is_annex_repo:
+        if data != "nothing":
+            log_progress(
+                lgr.info, pbar_id, "Transfer data",
+                label="Transfer data to '{}'".format(target), update=2, total=4)
+            yield from _push_data(
+                ds,
+                target,
+                content,
+                data,
+                force,
+                jobs,
+                res_kwargs.copy(),
+                got_path_arg=got_path_arg,
+            )
+        else:
+            lgr.debug("Data transfer to '%s' disabled by argument", target)
+    else:
         lgr.debug("No data transfer: %s is not a git annex repository", repo)
-        return
-
-    if data == "nothing":
-        lgr.debug("Data transfer to '%s' disabled by argument", target)
-        return
-
-    log_progress(
-        lgr.info, pbar_id, "Transfer data",
-        label="Transfer data to '{}'".format(target), update=2, total=4)
-
-    yield from _push_data(
-        ds,
-        target,
-        content,
-        data,
-        force,
-        jobs,
-        res_kwargs.copy(),
-        got_path_arg=got_path_arg,
-    )
 
     if not target_is_git_remote:
         # there is nothing that we need to push or sync with on the git-side
@@ -602,6 +591,9 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
         lgr.info, pbar_id, "Update availability information",
         label="Update availability for '{}'".format(target), update=3, total=4)
 
+    # TODO fetch is only needed if anything was actually transferred. Collect this
+    # info and make the following conditional on it
+
     # after file transfer the remote might have different commits to
     # the annex branch. They have to be merged locally, otherwise a
     # push of it further down will fail
@@ -643,29 +635,29 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
         if "fatal: couldn't find remote ref git-annex" not in e.stderr.lower():
             raise
         lgr.debug('Remote does not have a git-annex branch: %s', e)
-    # and push the annex branch to announce local availability info
-    # too
+
+    if is_annex_repo:
+        refspecs2push += [
+            'git-annex'
+            if ds.config.get('branch.git-annex.merge', None)
+            else 'git-annex:git-annex']
+    # and push all relevant branches, plus the git-annex branch to announce
+    # local availability info too
     yield from _push_refspecs(
         repo,
         target,
-        ['git-annex'
-         if ds.config.get('branch.git-annex.merge', None)
-         else 'git-annex:git-annex'],
+        refspecs2push,
         force_git_push,
         res_kwargs.copy(),
     )
 
 
 def _push_refspecs(repo, target, refspecs, force_git_push, res_kwargs):
-    # TODO inefficient, but push only takes a single refspec at a time
-    # at the moment, enhance GitRepo.push() to do all at once
-    push_res = []
-    for refspec in refspecs:
-        push_res.extend(repo.push(
-            remote=target,
-            refspec=refspec,
-            git_options=['--force'] if force_git_push else None,
-        ))
+    push_res = repo.push(
+        remote=target,
+        refspec=refspecs,
+        git_options=['--force'] if force_git_push else None,
+    )
     # TODO maybe compress into a single message whenever everything is
     # OK?
     for pr in push_res:
